(** S2DLDraws
@Name: Simple 2D Library - Simple GUI Components
  This file is part of the Free Pascal Simple 2D Libraries
@Version: 1.11
@Change_History:
@   ver 1.11 - Working on Linux (Daniel Mantione)
@   ver 1.10 - Created base logic
@Copyright: (c) 2004-2007 by William Cairns (CairnsGames)

@License: This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
       (I got this from the JEDI-SDL files and it sounds Cool!)
@
By using this work you have immediatly agreed with the above.
@
 The work can be used for any purpose. You may sell it, use it in a game
 delete it or change it as you wish with no reward to the the original
 developers.
@
 We do however ask that if this work is used that some mention of it be made
 on the PGDev web site.

@Usage: (To be completed)

@Author: W.Cairns **)
unit S2DLGuis;

{$ifdef fpc}
  {$mode objfpc}
{$endif}

{$h+}

interface

uses
  Classes, SysUtils, S2DLNeheFont, S2DLImage;
  
Type
  TS2DLControl = Class
  Private
    FActive : Boolean;
    FHeight : Integer;
    FLeft : Integer;
    FName : String;
    FTop : Integer;
    FWidth : Integer;
    FMouseIsOver: Boolean;
    FOnMouseOver: TNotifyEvent;
    FOnLeftClick: TNotifyEvent;
    FOnRightClick: TNotifyEvent;
    FVisible: Boolean;
    procedure SetActive(const AValue : Boolean);
    procedure SetMouseIsOver(const Value: Boolean);
    procedure SetOnLeftClick(const Value: TNotifyEvent);
    procedure SetOnMouseOver(const Value: TNotifyEvent);
    procedure SetOnRightClick(const Value: TNotifyEvent);
    procedure SetVisible(const Value: Boolean);
  Public
    Property Active : Boolean read FActive write SetActive;
    Property Top : Integer read FTop write FTop;
    Property Left : Integer read FLeft write FLeft;
    Property Width : Integer read FWidth write FWidth;
    Property Height : Integer read FHeight write FHeight;
    Property Name : String read FName write FName;
    Property OnLeftClick : TNotifyEvent read FOnLeftClick write SetOnLeftClick;
    Property OnRightClick : TNotifyEvent read FOnRightClick write SetOnRightClick;
    Property OnMouseOver : TNotifyEvent read FOnMouseOver write SetOnMouseOver;
    Property MouseIsOver : Boolean read FMouseIsOver write SetMouseIsOver;
    Property Visible : Boolean read FVisible write SetVisible;
    Constructor Create;
    Procedure MouseOver(X,Y : Integer; var Handled : Boolean); Virtual; Abstract;
    Procedure LeftClick(X,Y : Integer; var Handled : Boolean); Virtual; Abstract;
    Procedure RightClick(X,Y : Integer; var Handled : Boolean); Virtual; Abstract;
    Procedure Draw; Virtual; Abstract;
  End;
  TS2DLLabel = Class(TS2DLControl)
  Private
    FCaption : String;
    FFont : TS2DLNeheFont;
    procedure SetCaption(const AValue : String);
    procedure SetFont(const AValue : TS2DLNeheFont);
  Public
    Property Font : TS2DLNeheFont read FFont write SetFont;
    Property Caption : String read FCaption write SetCaption;
    Procedure LeftClick(X,Y : Integer; var Handled : Boolean); Override;
    Procedure RightClick(X,Y : Integer; var Handled : Boolean); Override;
    Procedure MouseOver(X,Y : Integer; var Handled : Boolean); Override;
    Procedure Draw; Override;
  End;
  TS2DLButton = Class(TS2DLLabel)
  Private
    FOffSetX, FOffSetY : Integer;
    FImage : TS2DLImage;
    procedure SetImage(const AValue : TS2DLImage);
  Public
    Property Image : TS2DLImage read FImage write SetImage;
    Property OffSetX : Integer read FOffSetX write FOffsetX;
    Property OffSetY : Integer read FOffSetY write FOffsetY;
    Procedure LeftClick(X,Y : Integer; var Handled : Boolean); Override;
    Procedure RightClick(X,Y : Integer; var Handled : Boolean); Override;
    Procedure MouseOver(X,Y : Integer; var Handled : Boolean); Override;
    Procedure Draw; Override;
  End;

  TS2DLControlList = Class(TS2DLControl)
  Private
    Items : TList;
  Public
    Constructor Create;
    Destructor Destroy;
    Procedure LeftClick(X,Y : Integer; var Handled : Boolean); Override;
    Procedure RightClick(X,Y : Integer; var Handled : Boolean); Override;
    Procedure MouseOver(X,Y : Integer; var Handled : Boolean); Override;
    Procedure Draw; Override;
    Procedure AddLabel(Font : TS2DLNeheFont; X,Y : Integer; Caption : String);
    Function AddButton(Font : TS2DLNeheFont; X,Y : Integer; Caption : String;
                        Image : TS2DLImage; OffSetX,OffSetY : Integer) : TS2DLButton;
    Function AddGroup(GroupName : String) : TS2DLControlList;
    Function ByName(AName : String) : TS2DLControl;
  End;

implementation

{ TS2DLControlList }

constructor TS2DLControlList.Create;
begin
  Inherited Create;
  Items := TList.Create;
end;

destructor TS2DLControlList.Destroy;
Var
  I : Integer;
Begin
  // Free memory used for each S2DLControl
  For I := (Items.Count-1) downto 0 do
    Begin
      TS2DLControl(Items[I]).Free;
      Items.Delete(I);
    End;
  Items.Free;
end;

procedure TS2DLControlList.LeftClick(X, Y : Integer; var Handled : Boolean);
Var
  I : Integer;
begin
  If Not Active then
    Exit;
  For I := 0 to Items.Count - 1 do
    If Not Handled then
      TS2DLControl(Items[I]).LeftClick(X,Y,Handled);
end;

procedure TS2DLControlList.MouseOver(X, Y : Integer; var Handled : Boolean);
Var
  I : Integer;
begin
  If Not Active then
    Exit;
  For I := 0 to Items.Count - 1 do
    TS2DLControl(Items[I]).MouseOver(X,Y,Handled);
end;

procedure TS2DLControlList.Draw;
Var
  I : Integer;
begin
  For I := 0 to Items.Count - 1 do
    If TS2DLControl(Items[I]).Active then
      TS2DLControl(Items[I]).Draw;
end;

procedure TS2DLControlList.AddLabel(Font : TS2DLNeheFont; X, Y : Integer;
  Caption : String);
Var
  C : TS2DLLabel;
begin
  C := TS2DLLabel.Create;
  C.Font := Font;
  C.Left := X;
  C.Top := Y;
  C.Caption := Caption;
  Items.Add(C);
end;

Function TS2DLControlList.AddButton(Font : TS2DLNeheFont; X, Y : Integer;
  Caption : String; Image : TS2DLImage; OffSetX,OffSetY : Integer) : TS2DLButton;
Var
  C : TS2DLButton;
begin
  C := TS2DLButton.Create;
  C.Font := Font;
  C.Left := X;
  C.Top := Y;
  C.Caption := Caption;
  C.Image := Image;
  C.OffSetX := OffsetX;
  C.OffSetY := OffsetY;
  Items.Add(C);
  Result := C;
end;

Function TS2DLControlList.AddGroup(GroupName : String) : TS2DLControlList;
Var
  C : TS2DLControlList;
begin
  C := TS2DLControlList.Create;
  C.Name := GroupName;
  Items.Add(C);
  Result := C;
end;

function TS2DLControlList.ByName(AName : String) : TS2DLControl;
Var
  I : Integer;
  C : TS2DLControl;
begin
  Result := Nil;
  For I := 0 to Items.Count - 1 do
    Begin
      C := TS2DLControl(Items[I]);
      If C.Name = NAme then
        Begin
          Result := C;
          Exit;
        End
      Else
        Begin
          If C is TS2DLControlList then
            Begin
              C := TS2DLControlList(C).ByName(Name);
              If C <> Nil then
                Begin
                  Result := C;
                  Exit;
                End;
            End;
        End;
    End;
end;

procedure TS2DLControlList.RightClick(X, Y: Integer; var Handled : Boolean);
Var
  I : Integer;
begin
  If Not Active then
    Exit;
  For I := 0 to Items.Count - 1 do
    If Not Handled then
      TS2DLControl(Items[I]).LeftClick(X,Y,Handled);
end;

{ TS2DLControl }

procedure TS2DLControl.SetActive(const AValue : Boolean);
begin
  if FActive = AValue then
    exit;
  FActive := AValue;
end;

constructor TS2DLControl.Create;
begin
  Active := True;
  Visible := True;
end;

procedure TS2DLControl.SetMouseIsOver(const Value: Boolean);
Var
  WasValue : Boolean;
begin
  If FMouseIsOver <> Value then
    Begin
      WasValue := FMouseIsOver;
      FMouseIsOver := Value;
      If MouseIsOver and (WasValue <> FMouseIsOver) then
        If Assigned(OnMouseOver) then
          OnMouseOver(Self);
    End;
end;

procedure TS2DLControl.SetOnLeftClick(const Value: TNotifyEvent);
begin
  FOnLeftClick := Value;
end;

procedure TS2DLControl.SetOnMouseOver(const Value: TNotifyEvent);
begin
  FOnMouseOver := Value;
end;

procedure TS2DLControl.SetOnRightClick(const Value: TNotifyEvent);
begin
  FOnRightClick := Value;
end;

procedure TS2DLControl.SetVisible(const Value: Boolean);
begin
  FVisible := Value;
end;

{ TS2DLLabel }

procedure TS2DLLabel.SetCaption(const AValue : String);
begin
  if FCaption = AValue then
    Exit;
  FCaption := AValue;
  FName := FCaption;
end;

procedure TS2DLLabel.SetFont(const AValue : TS2DLNeheFont);
begin
  if FFont = AValue then
    exit;
  FFont := AValue;
end;

procedure TS2DLLabel.LeftClick(X, Y : Integer; var Handled : Boolean);
begin
end;

procedure TS2DLLabel.MouseOver(X, Y : Integer; var Handled : Boolean);
begin
end;

procedure TS2DLLabel.Draw;
begin
  Font.RenderFont(Left,Top,Caption);
end;

procedure TS2DLLabel.RightClick(X, Y: Integer; var Handled : Boolean);
begin
end;

{ TS2DLButton }

procedure TS2DLButton.SetImage(const AValue : TS2DLImage);
begin
  Width := AValue.PatternWidth;
  Height := AValue.PatternHeight;
  FImage := AValue;
end;

procedure TS2DLButton.MouseOver(X, Y : Integer; var Handled : Boolean);
begin
  If Not Visible then
    Exit;
  If (X > Left) and (X < Left+Width) and
     (Y > Top) and (Y < Top+Height) and (Not Handled)then
    Begin
      MouseIsOver := True;
      Handled := True;
    End
  Else
    MouseIsOver := False;
end;

procedure TS2DLButton.LeftClick(X, Y : Integer; var Handled : Boolean);
begin
  If Not Visible then
    Exit;
  If (X > Left) and (X < Left+Width) and
     (Y > Top) and (Y < Top+Height) then
    If Assigned(OnLeftClick) then
      Begin
        OnLeftClick(Self);
        Handled := True;
      End;
end;

procedure TS2DLButton.Draw;
begin
  If Not Visible then
    Exit;
  If MouseIsOver then
    Image.DrawStretch(Left,Top,Width,Height,1)
  else
    Image.DrawStretch(Left,Top,Width,Height,0);
  Font.Write(Left+OffSetX,Top+OffSetY,Caption);
end;

procedure TS2DLButton.RightClick(X, Y: Integer; var Handled : Boolean);
begin
  If Not Visible then
    Exit;
  If (X > Left) and (X < Left+Width) and
     (Y > Top) and (Y < Top+Height) then
    If Assigned(OnRightClick) then
      Begin
        OnRightClick(Self);
        Handled := True;
      End;
end;

end.

