(**S2DLLogger
@Name: Simple 2D Library - Logger Unit
@  This file is part of the Free Pascal Simple 2D Libraries
@  This file based on the Logger unit from JEDI-SDL
@ Changes for S2DLLogger unit
@   Set bedug level to control level of log detail
@   Auto check config file for debug level
@      If no config file found then uses highest level
@    0 = No Logging
@    1 = Errors only
@    2 = Errors and Warnings
@    3 = Errors, Warning and Status messages = All
@Note: Do not initialize yourself !!! - Automatically done !!!
@Purpose: Manages Logging Functionality for S2DL as well as standard Configuration
@ Setting information.
@Version: 1.11
@Change_History: 
@   ver 1.11 - Working on Linux (Daniel Mantione)
@   ver 1.10 - Added SoundOn and FullScreen methods to check config file.
@Copyright: (c) 2004-2007 by William Cairns (CairnsGames) 

@License: This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
       (I got this from the JEDI-SDL files and it sounds Cool!)
@
By using this work you have immediatly agreed with the above.
@
 The work can be used for any purpose. You may sell it, use it in a game
 delete it or change it as you wish with no reward to the the original
 developers.
@
 We do however ask that if this work is used that some mention of it be made
 on the PGD web site.

@Usage: Call Log.LogError or Log.LogWarning or Log.LogStatus to create a line
@ in the Log

@Author: W.Cairns
  
**)
unit S2DLLogger;

{$ifdef FPC}
{$mode delphi}
{$endif}

Interface

uses
  Classes,
  SysUtils;
  
type
  (**
     TLogger is a psuedo class that manages all the Logging functionality and also stores
     a copy of the game config file (if it exists).
  **)
  TLogger = class(TFileStream)
  private
    FApplicationName : string;
    FApplicationPath : string;
    FLogLevel : Integer;
    Config : TStringList;
    procedure SetLogLevel(const AValue : Integer);
  protected
  public
    constructor Create;
    procedure LogError( ErrorMessage : string; Location : string );
    procedure LogWarning( WarningMessage : string; Location : string );
    procedure LogStatus( StatusMessage : string; Location : string );
    procedure CreateFile;
    Procedure LoadConfig;
  published
    property ApplicationName : string read FApplicationName;
    property ApplicationPath : string read FApplicationPath;
    property LogLevel : Integer read FLogLevel write SetLogLevel;
  end;

var
  Log : TLogger;

Function SoundOn : Boolean;
Function FullScreen : Boolean;

implementation

(**
  FullScreen checks the Application Configuration file (automatically) to see
  if the program must run as a full screen application.

  @return  a Boolean
**)
Function FullScreen : Boolean;
Var
  V : String;
  R : Boolean;
Begin
  V := Log.Config.Values['FullScreen'];
  R := False;
  If V = '' then
    V := 'N';
  If V[1] in ['Y','y'] then
    R := True;
  Result := R;
End;

(*Var
  R : Boolean;
  RS : String;
  FApplicationPath : String;
  FApplicationName : String;
  N : String;
  L : TStringList;
Begin
  R := False;
  FApplicationName := ExtractFileName( ParamStr(0) );
  FApplicationPath := ExtractFilePath( ParamStr(0) );
  N := FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.cfg';
  If FileExists(N) then
    begin
      L := TStringList.Create;
      L.LoadFromFile(FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.cfg');
      RS := L.Values['FullScreen'];
      If RS = '' then
        RS := 'N';
      If RS[1] in ['Y','y'] then
        R := True;
      L.Free;
    End;
  Result := R;
End;       *)

(**
  SoundOn checks the Application Configuration file (automatically) to see
  if the program must run with or without sound.

  @return  a Boolean
**)
Function SoundOn : Boolean;
Var
  V : String;
  R : Boolean;
Begin
  V := Log.Config.Values['Sound'];
  R := False;
  If V = '' then
    V := 'N';
  If V[1] in ['Y','y'] then
    R := True;
  Result := R;
End;
(*
Var
  R : Boolean;
  RS : String;
  FApplicationPath : String;
  FApplicationName : String;
  N : String;
  L : TStringList;
Begin
  R := False;
  FApplicationName := ExtractFileName( ParamStr(0) );
  FApplicationPath := ExtractFilePath( ParamStr(0) );
  N := FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.cfg';
  If FileExists(N) then
    begin
      L := TStringList.Create;
      L.LoadFromFile(FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.cfg');
      RS := L.Values['Sound'];
      If RS = '' then
        RS := 'N';
      If RS[1] in ['Y','y'] then
        R := True;
      L.Free;
    End;
  Result := R;
End;*)

{ TLogger }
(**
  This is the constructor method for the TLogger class. Called Automatically at
  the start of the program. 
**)
constructor TLogger.Create;
Var
  V : String;
begin
  FApplicationName := ExtractFileName( ParamStr(0) );
  FApplicationPath := ExtractFilePath( ParamStr(0) );
  LogLevel := 0;
  If Config = Nil then
    LoadConfig;
  V := Config.Values['Debug'];
  If V = '' then
    V := '3';
  If Not (V[1] in ['0','1'..'3']) then
    V := '3';
  LogLevel := StrToInt(V);
end;

(**
  This is a setter method for the LogLevel property.
  Sets the Log Level
@    0 = No Logging
@    1 = Errors only
@    2 = Errors and Warnings
@    3 = Errors, Warning and Status messages = All

  @param   AValue as an Integer constant

**)
procedure TLogger.SetLogLevel(const AValue : Integer);
begin
  if FLogLevel=AValue then exit;
  If FLogLevel = 0 then
    CreateFile;
  FLogLevel:=AValue;
end;

(**
  Creates the Config File.
**)
procedure TLogger.CreateFile;
begin
  inherited Create( FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.log',
                    fmCreate {$IFNDEF FPC}or fmShareExclusive{$ENDIF} );
end;

(**
  Logs an Error Message. Will be done for all levels of LogLevel above 0 (No Logging).

  @param   ErrorMessage as a string
  @param   Location     as a string
**)
procedure TLogger.LogError(ErrorMessage, Location: string);
var
  S : string;
begin
  If LogLevel > 0 then
    Begin
      S := '*** ERROR *** : @ ' + TimeToStr(Time) + ' MSG : ' + ErrorMessage + ' IN : ' + Location + #13#10;
      Write(S[1], Length(S));
    End;
end;

(**
  Logs a Status Message. Will be done for all levels of LogLevel above 2.

  @param   StatusMessage as a string
  @param   Location     as a string
**)
procedure TLogger.LogStatus(StatusMessage, Location: string);
var
  S : string;
begin
  If LogLevel > 2 then
    Begin
      S := 'STATUS INFO : @ ' + TimeToStr(Time) + ' MSG : ' + StatusMessage + ' IN : ' + Location + #13#10;
      Write(S[1], Length(S));
    End;
end;

(**
  Logs a Warning Message. Will be done for all levels of LogLevel above 1.

  @param   WarningMessage as a string
  @param   Location     as a string
**)
procedure TLogger.LogWarning(WarningMessage, Location: string);
var
  S : string;
begin
  If LogLevel > 1 then
    Begin
      S := '=== WARNING === : @ ' + TimeToStr(Time) + ' MSG : ' + WarningMessage + ' IN : ' + Location + #13#10;
      Write(S[1], Length(S));
    End;
end;

(**
  Loads the config file into memeory for quicker access by later calls.
  The Config can be access using Log.Config.Values.
**)
procedure TLogger.LoadConfig;
begin
  If Not Assigned(Config) then
    Config := TStringList.Create;
  If FileExists(FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.cfg') then
    begin
      Config.LoadFromFile(FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.cfg');
    end;
end;

initialization
begin
  Log := TLogger.Create;
  Log.LogStatus( 'Starting Application', 'Initialization' );
end;

finalization
begin
  Log.LogStatus( 'Terminating Application', 'Finalization' );
  Log.Free;
  Log := nil;
end;

end.






