{** This unit defines classes that wraps opengl textures.
A loading function to load textures from png is provided. }
unit textureunit;

{$ifdef fpc}
  {$mode objfpc}
{$endif}

{$h+}

interface
uses
  Classes, SysUtils,
  GL, sdl, SDL_Image;

type
     TTexture = class
      name:String;
      texture:GLuInt;
      Width, Height : Integer;
      procedure LoadFrom(filename:string);
      procedure Bind;
      destructor Destroy;override;
     end;

Var
  FontTexture : TTexture;

implementation

function LoadGLTexture(Caller : TTexture; filename: string; var TextureID : GLuInt ): Boolean;
var
  // Create storage space for the texture
  TextureImage: PSDL_Surface;
begin
  // Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
  TextureImage := IMG_Load(pchar(filename));
  
  if ( TextureImage <> nil ) then
  begin
    result:=true;

    Caller.Width := TextureImage^.w;
    Caller.Height := TextureImage^.h;
    // Create Texture
    glGenTextures( 1, @TextureID );
    // Typical Texture Generation Using Data From The Bitmap
    glBindTexture( GL_TEXTURE_2D, TextureID );

    // Generate The Texture
    with TextureImage^ do
      if format^.bitsperpixel=8 then
        glTexImage2D( GL_TEXTURE_2D, 0, GL_ALPHA, w, h, 0, GL_ALPHA, GL_UNSIGNED_BYTE, pixels )
      else if format^.bitsperpixel=24 then
        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, pixels )
      else
        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, pixels );

    // Linear Filtering
    // scale linearly when image bigger than texture
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    // scale linearly when image smaller than texture
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  end
  else
  begin
   raise exception.create('file '+filename+' not found');
   result:=false;
  end;

  // Free up any memory we may have used
  if ( TextureImage <> nil ) then
    SDL_FreeSurface( TextureImage );
end;


procedure TTexture.Bind;
begin
  glBindTexture(GL_TEXTURE_2D, texture);
end;

destructor TTexture.Destroy;
begin
	glDeleteTextures(1, @texture );
	inherited Destroy;
end;

procedure TTexture.LoadFrom(filename:string);
begin
  LoadGLTexture(Self, filename, texture);
end;

end.
 
